/*========================================================================================================================
This version of the software is for the paper "Itchy Feet vs Cool Heads: Flow of Funds in an Agent-based Financial Market".

The software can be compiled based on the following 5 files: model.cc, model_flow.cc, investors.cc, dividend.cc, and main.cc (or main_for_statistics.cc).

model.cc solves the model without the flow of funds;
model_flow.cc solves the model with the flow of funds;
investors.cc computes investment proportions for the three fund types;
dividend.cc generates truncated normal dividends;
main.cc is used to specify model parameters and output time series of the price, wealth, investment proportions, performance measures and population;
main.cc can be replaced by "main_for_statistics.cc" if the purpose is to get summary statistics of the time series based on X>1 independent runs. E.g. to get results in Table 2-4 in the paper.

The goal of the software is to facilitate the corresponding research, the code is written in a way that allows esay modificatation, computational efficiency (memory usage and speed) is not our primary concern when writting the code.
The software is compatible with the multi-asset framework (Section 2.1 of the paper).
/*=================================================================================
This file contains: 1) The main function main()
                       Aims: Declare variables, arrays.
					         Output results in .txt file.

                    2) The function input_data()
					   Aims: Provide a user input interface.
					         Check whether the data inputed is correct.
==================================================================================*/
#include <iostream>
#include <cmath>
#include <time.h>
#include <fstream>
using namespace std;

void input_data (double& T, int& N, int& m, double& T_1); //user input interface

void price_matrix (double *price, double *dividend, double *market_return, double niu, int num_obs, int m,double **pri_matrix);//The price for each risky asset during observation time T_1

void compute_fundamental (double **pri_matrix, int num_obs, double r, double *lamda_F_1); //Fundamental trader's investment proportions

void compute_noise (double& noise_intensity, double *lamda_N_1); //Noise trader's investment proportions

void compute_trend (double *market_return, int num_obs,  double r, double *lamda_T_1);//trend follower's investment porportions

void compute_dividend (double *dividend);//dividend for each risky asset
//compute wealth and price
void compute_wealth(double niu, double c, double r,int m, int num_obs, double **lamda, double *wealth, double **portfolio, double **pri_matrix, double *price, double *bank, double& noise_intensity, double *dividend, double *lamda_F_1,double *lamda_T_1,double *lamda_N_1, double *market_return);
void compute_wealth_flow(double beta, double niu, double c, double r,int m, double& totalwealth, int num_obs, double **lamda, double *wealth, double *population, double *pre_population, double **portfolio, double **pri_matrix, double *price,double *bank, double& noise_intensity,double *dividend, double *lamda_F_1,double *lamda_T_1,double *lamda_N_1, double *market_return);


double normal_variate (void);
int main ()
{
   double T,T_1; //T is the totol number of periods for the simulation, T_1 is investors' observation horizon
   int N,m,number_cut,number_divide,number_av;//N is the number of trades during T (For D2C version model use only). m is number of risky assets.
   double total,sum,totalwealth,noise_intensity;
   double rho; //rho controls the degree of recency bias in performance evaluation
   double r;// r is risk-free interest rate

   
   double vol, total_vol, sum_vol;//volume
   double port_temp[3];


   input_data (T, N, m, T_1);

   double niu,c;
     r=0.001;
	 c=0.000999000999000999;
	
   //niu = T/(double)N; // length of time between two successive trades
	 niu = 1; //for standard version model
	 noise_intensity = 0;//initialising the noise trader
   
//memory allocation for investment proportion matrix (size K x I)
   double** lamda = new double*[m+1];
   for (int i=0; i<m+1; i++)
	   lamda[i]= new double[3]; 



  double* bank = new double[3]; // portfolios of the risk-free asset;

//memory allocation for portfolio matrix (size I x K)
   double** portfolio = new double*[3];
   for (int i=0; i<3; i++)
	   portfolio[i]= new double[m];

//memory allocation for investors' wealth
   double* wealth = new double[3];
 
	     wealth[0]=667;     //investors' inital wealth
	     wealth[1]=667;
		 wealth[2]=667;
		 
//memory allocation for dividends
   double* dividend = new double[m];

//memory allocation for observed price data set
   int num_obs=0;
   while(num_obs<=(T_1/niu)) num_obs++;
   
   double** pri_matrix = new double*[m];
    for (int i=0; i<m; i++)
	   pri_matrix[i]= new double[num_obs]; 

	 double* market_return = new double[num_obs-1];
   for (int i=0; i<num_obs-1; i++) 
	   market_return[i]=0;

   double* price = new double[m]; // prices 
	  
   double* lamda_F_1 = new double[m+1]; //investment proportions for the fundamental trader

   double* lamda_T_1 = new double[m+1];//trend follower

   double* lamda_N_1 = new double[m+1];//noise trader



	 double* population= new double [3];
	 double* pre_population= new double [3];
	 double w_return[3];
     double wealth_temp[3];
	 double invest_temp[3];
	
	double** fitness = new double*[3];
    for (int i=0; i<3; i++)
	   fitness[i]= new double[N];
	fitness[0][0]=0;
	fitness[1][0]=0;
	fitness[2][0]=0;


	double gamma;
	gamma=2;
	rho=0.99;

	double beta; // beta=1 is flow of funds, beta=0 no flow of funds
	beta=1.0;

	number_cut=1000000;
	number_divide = N - number_cut;
	  	

	srand((unsigned)time(NULL)); //initialise the random number generator 
	//srand(1);
    price_matrix (price,dividend,market_return,niu,num_obs,m,pri_matrix); //get stock prices for each stock during observation time T_1


    ofstream myfile ("wealth.txt");//output into .txt files
	ofstream myfile2 ("dividend.txt");
	ofstream myfile3 ("AssetPrice.txt");
	ofstream myfile4 ("investment_proportions.txt");
	ofstream myfile5 ("performance.txt");
	ofstream myfile6 ("fof_population.txt");	
 if (myfile.is_open() && myfile2.is_open() && myfile3.is_open() && myfile4.is_open() && myfile5.is_open() && myfile6.is_open())//check whether the files can be created
 {


	 compute_dividend (dividend);

	  for (int j=0; j<m; j++) // output dividends in dividend.txt
			 myfile2 <<dividend[j]<<" ";
					
	 compute_fundamental (pri_matrix,num_obs,r,lamda_F_1);
	  for (int i=0; i<m+1; i++)
		 lamda[i][1]=lamda_F_1[i];
						


	 compute_noise (noise_intensity, lamda_N_1);
	   for (int i=0; i<m+1; i++)
		 lamda[i][0]=lamda_N_1[i];
					

	 compute_trend (market_return, num_obs, r, lamda_T_1);
	   for (int i=0; i<m+1; i++)
		 lamda[i][2]=lamda_T_1[i];
					 						 
/* Now investors investment proportions matrix has been assembled completely! */
	   //output investment proportions
	    for (int j=0; j<m; j++) 
			     myfile4 <<lamda[j][0]<<" ";
			  for (int j=0; j<m; j++)  
			     myfile4 <<" "<<lamda[j][1]<<" ";
			   for (int j=0; j<m; j++)  
			     myfile4 <<" "<<lamda[j][2]<<" ";
				 myfile4 <<" "<<endl;


	   for (int i=0; i<3; i++)
			invest_temp[i] = lamda[0][i];
						  
//compute results for N periods ==========================================			
				 for(int i=0; i<N; i++)
				 { 
					total=0;
					sum=0;
						     
					  if (i==0)
					  {


						  for (int j=0; j<3; j++)
						   wealth_temp[j]=(1-c)*wealth[j];

					     compute_wealth(niu,c,r,m, num_obs, lamda, wealth, portfolio, pri_matrix, price,bank, noise_intensity,dividend,lamda_F_1,lamda_T_1,lamda_N_1,market_return);//get investors wealth
		               
		                 
						
						for (int j=0; j<3; j++)
						{
						   w_return[j]=wealth[j]/wealth_temp[j];
					       w_return[j]=log(w_return[j]);
						}
						
				
					    
						for(int j = 0; j < 3; j++)
							total += wealth[j];
						for(int j = 0; j < 3; j++)
						{
							pre_population[j] = wealth[j]/total;
							port_temp[j]=portfolio[j][0];
						}


					   }//end if i is 0



		       
					 if(i>0)
					 {
						 if (beta==0)
						  {
								for (int j=0; j<3; j++)
								  wealth_temp[j]=(1-c)*wealth[j];

							  compute_wealth(niu,c,r,m, num_obs, lamda, wealth, portfolio, pri_matrix, price,bank,noise_intensity,dividend,lamda_F_1,lamda_T_1,lamda_N_1,market_return);//get investors wealth
			               
							   for (int j=0; j<3; j++)
							    {
						          w_return[j]=wealth[j]/wealth_temp[j];
					              w_return[j]=log(w_return[j]);
						        }
										 
						   }else if( beta>0){

							   for (int j=0; j<3; j++)
								  wealth_temp[j]=wealth[j];

							total = 0;
							
							for (int j=0; j<3; j++)
							{
							 fitness[j][i]= w_return[j] + rho*fitness[j][i-1];
							}

							for(int j = 0; j < 3; j++)
							  total += wealth[j];

		                     
							for(int j = 0; j < 3; j++)
						       {
							       for(int k=0; k<3; k++)
								     sum += exp(gamma*(fitness[k][i]-fitness[j][i]));

						              population[j]= 1/sum;

						            sum=0;
						       }

													  
							totalwealth = total;

							compute_wealth_flow(beta,niu, c, r, m, totalwealth, num_obs, lamda, wealth, population, pre_population, portfolio, pri_matrix, price,bank, noise_intensity,dividend, lamda_F_1,lamda_T_1,lamda_N_1,market_return);

							 
							 for (int j=0; j<3; j++)
							 {
								 wealth_temp[j]=(1-c)*(beta*pre_population[j]*totalwealth + (1-beta)*wealth_temp[j]);
							
						         w_return[j]=wealth[j]/wealth_temp[j];
					             w_return[j]=log(w_return[j]);
					
								 port_temp[j]= fabs(portfolio[j][0]-port_temp[j]);
							 
							 }


							 total_vol=0;
							 vol=0;
							 for (int j=0; j<3; j++)
							  total_vol += port_temp[j];
							 vol= total_vol/2;
							 

							 

			                 
							 for (int j=0; j<3; j++)
							 {
								 pre_population[j] = population[j];
								 port_temp[j]=portfolio[j][0];	
							 }
						  }//end if beta>0
					  }//end if i>0
		           
					 total=0;


					 for (int j=0; j<3; j++)//compute total wealth
						total += wealth[j]; 

					 
		  if (i >= number_cut)//------------------cut initial number_cut periods-------------------------------
		  {

			myfile <<wealth[0]<<" "<<wealth[1]<<" "<<wealth[2]<<" "<<total<<" "<<wealth[0]/total<<" "<<wealth[1]/total<<" "<<wealth[2]/total<<" "<<endl;// output relative wealth in wealth.txt
			 
			 for (int j=0; j<m; j++) // output dividends in dividend.txt
			 {   myfile2 <<dividend[j]<<" ";
				 myfile3 <<price[j]<<" ";
			 }
			 myfile2 <<" "<<endl;
			 myfile3 <<" "<<endl;

			
			 
           //two asset, 0 is risky, 1 is risk-free
			  for (int j=0; j<m; j++)  
			     myfile4 <<lamda[j][0]<<" ";
			  for (int j=0; j<m; j++)  
			     myfile4 <<" "<<lamda[j][1]<<" ";
			    for (int j=0; j<m; j++)  
			     myfile4 <<" "<<lamda[j][2]<<" ";

				 myfile4 <<" "<<endl;


			 myfile5 <<fitness[0][i]<<" "<<fitness[1][i]<<" "<<fitness[2][i]<<" "<<endl;

				for (int j=0; j<3; j++) 
			    {  
				    myfile6 <<pre_population[j]<<" ";
					
			    }
			     myfile6 <<" "<<endl;            

		 }//end if i>N-x;
	 }//end for N

     
		   
        myfile.close();//save and close wealth.txt
		cout << "wealth.txt has been created"<<endl;
		myfile2.close();
		cout << "dividend.txt has been created"<<endl;
		myfile3.close();
		cout << "AssetPrice.txt has been created"<<endl;
		myfile4.close();
		cout << "Investment_proportions.txt has been created"<<endl;
		myfile5.close();
		cout << "performance.txt has been created"<<endl;
		myfile6.close();
		cout << "fof_population.txt has been created"<<endl;
	  } else cout << "Unable to open file";

		  
	 int p;
  cin>>p;
  return 0;
}

void input_data (double& T, int& N, int& m, double& T_1)
{

  cout<<"Please enter the number of periods in the simulation:"<<endl;
  cin>>T; 
  while (T<=0)//check whether the time is negative or 0,if so, ask user re-input
    {
	  cout<<"You enter invailid time horizon! It must be positive!"<<endl;
      cout<<"Please enter positive time horizon again!"<<endl;
      cin>>T;
    }

//for D2C version model
/* 
  cout<<"Please enter the number of trades during the time horizon:"<<endl;
  cin>>N; 
  while (N<=0)//check whether the number is negative or 0, if so, ask user re-input
    {
	   cout<<"number of trades must be positive!"<<endl;
       cout<<"Please enter the number of trades again:"<<endl;
       cin>>N;
    }
*/ 
N=T;//for standard version with the length of time step = 1.

//for multi-asset model
/*
   cout<<"Please enter the number of stocks in the market:"<<endl;
  cin>>m; 
    while (m<2)//check whether the number of stocks is less than 2, if so, ask user re-input
    {
	   cout<<"number of stocks has to be greater than 1!"<<endl;
       cout<<"Please enter the number of stocks again:"<<endl;
       cin>>m;
    }
*/
m=1;//single risky asset model

  cout<<"Please enter the observation time:"<<endl;
  cin>>T_1; 
  while (T_1<=0)//check whether the time is negative or zero, if so, ask user re-input
    {
	   cout<<"Observation time must be positive!"<<endl;
       cout<<"Please enter the observation time again:"<<endl;
       cin>>T_1;
    }

}







